"""Added polyline start/end types

Peek Plugin Database Migration Script

Revision ID: e8d6d02c0922
Revises: 6d3a96cf1955
Create Date: 2019-06-03 11:47:15.090106

"""

# revision identifiers, used by Alembic.
revision = 'e8d6d02c0922'
down_revision = '6d3a96cf1955'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('DispPolyline', sa.Column('endEndType', sa.Integer(), nullable=True), schema='pl_diagram')
    op.add_column('DispPolyline', sa.Column('startEndType', sa.Integer(), nullable=True), schema='pl_diagram')
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('DispPolyline', 'startEndType', schema='pl_diagram')
    op.drop_column('DispPolyline', 'endEndType', schema='pl_diagram')
    # ### end Alembic commands ###